# Definition of Done for Simantics Platform Releases

1. The `simantics/platform` and `simantics/third-party` Git repositories have a branch `release/x.y.z[.w]` and tag `x.y.z[.w]`.
2. [A change log entry](https://www.simantics.org/redmine/projects/simantics-platform/wiki/ChangeLog) is compiled from the issues in this release and made available to the general public separately for the platform and for the open products included in the release train.
3. [Roadmap](http://dev.simantics.org/index.php/Roadmap) is up-to-date.
4. [Tutorials](http://dev.simantics.org/index.php/Tutorials) are up-to-date and coherent with the platform.
5. For all new major/minor releases, Wiki documentation is backed up (cloned).

----

# Simantics Platform Release Process

* Create `release/x.y.z[.w]` release stabilisation branch
* Repeat until stable:
  * Develop, test and document components and test products
* Tag the release in repository
* Backup documentation wiki databases
* Build and publish SDK package with and without sources
  * Does not really require any work
* Build Simantics open source products and plug-in components which are a part of the release train
* Update websites to reflect the new release

----

# Released Plug-in Components and Products  

There are both plug-in components and products that are part of the "Simantics release train" that shall be released simultaneously to a major or minor Simantics release.

Plug-in components are installable features that are deployed online as P2 repositories for general availability. Products are deployed as ZIP files and made available online in designated locations on simantics.org. 

Products that are part of the release train: 
* Simantics Desktop
* Simantics System Dynamics Tool - [simantics/sysdyn.git](https://www.simantics.org:8088/r/gitweb?p=simantics/sysdyn.git;a=summary)

Plug-in components that are part of the release train:
* Simantics R - [simantics/r.git](https://www.simantics.org:8088/r/gitweb?p=simantics/r.git;a=summary)
* FMIL - [simantics/fmil.git](https://www.simantics.org:8088/r/gitweb?p=simantics/fmil.git;a=summary)
* FMI Studio - [members/fmi.git](https://www.simantics.org:8088/r/gitweb?p=members/fmi.git;a=summary)
* Simupedia - [Members SVN](https://www.simantics.org/svn/members/simupedia)

For simplicity, each of these components are versioned accoring to platform versioning, i.e. for Platform SDK 1.26.0 there will be Simantics Desktop 1.26.0, Sysdyn 1.26.0, and so on.  

----

# Release Schedule

* Releases are branched 4 weeks before the set release date.
* During this time no more new features are allowed in the release branch and the existing work shall be stabilized. Work must be focused on bug fixes and documentation.
* Source code changes to release branches are forbidden in the last 2 weeks. Documentation-only changes are still allowed. However, if critical show-stopper problems surface during that time, the changes must be done. After that the release engineering team must decide whether to stick to the original release schedule or whether to delay the release.   

----

# Simantics Platform Release - Step by Step

## Create release branch from selected commit

When release stabilisation starts, branch `simantics/platform` and `simantics/third-party` repositories:

    git clone ssh://<user>@www.simantics.org:29418/simantics/platform.git
    cd platform
    git branch release/x.y.z[.w] <commit>
    git push origin release/x.y.z[.w]

    git clone ssh://<user>@www.simantics.org:29418/simantics/third-party.git
    cd third-party
    git branch release/x.y.z[.w] <commit>
    git push origin release/x.y.z[.w]

When creating major/minor releases `<commit>` is usually a commit in the `master` branch.
With service releases, branch from an existing `release/*` branch instead.

## Prepare release branch for use

### Prepare .target files

1. Retrieve release branch of the platform repository

       git clone ssh://<user>@www.simantics.org:29418/simantics/platform.git
       cd platform
       git branch release/x.y.z[.w] remotes/origin/release/x.y.z[.w]
       git checkout release/x.y.z[.w]

2. Edit all target platform files in  `releng/org.simantics.sdk.build.targetdefinition/`, i.e.
   * `simantics.target`
   * `org.simantics.sdk.build.targetdefinition.target`

   At the beginning of simantics.target file, increment `sequenceNumber` by 1 and replace
   the version numbers in target name and `org.simantics.sdk.feature.group` and
   `org.simantics.sdk.source.feature.group` with `x.y.z[.w]`:
   ~~~
   <?xml version="1.0" encoding="UTF-8" standalone="no"?>
   <?pde version="3.8"?>
   <target name="Simantics x.y.z[.w]" sequenceNumber="11">
   <unit id="org.simantics.sdk.feature.group" version="x.y.z[.w]"/>
   <unit id="org.simantics.sdk.source.feature.group" version="x.y.z[.w]"/>
   ~~~

   Next, replace the following rows in both mentioned files:   

   ~~~
   <repository location="http://www.simantics.org/download/master/sdk"/>
   <repository location="http://www.simantics.org/download/master/external-components/maven"/>
   <repository location="http://www.simantics.org/download/master/external-components/manual"/>
   ~~~

   with

   ~~~
   <repository location="http://www.simantics.org/download/release/x.y.z[.w]/sdk"/>
   <repository location="http://www.simantics.org/download/release/x.y.z[.w]/external-components/maven"/>
   <repository location="http://www.simantics.org/download/release/x.y.z[.w]/external-components/manual"/>
   ~~~

3. Edit version number of `org.simantics.sdk` feature in `features/org.simantics.sdk.feature/feature.xml` to `x.y.z[.w]`.
   ~~~
   <feature
         id="org.simantics.sdk"
         label="Simantics SDK"
         version="x.y.z"
         provider-name="VTT Technical Research Centre of Finland">
   ~~~

### Initialize release branch distribution web site

* Run [SDK/Deploy External Components to Web](https://www.simantics.org/jenkins/job/SDK/job/Deploy%20External%20Components%20to%20Web/) build with parameters:
  * **GERRIT_REFNAME:** `release/x.y.z[.w]`
  * **PUBLISH_ARTIFACTS:** `true`
* Run the [SDK/Simantics SDK](https://www.simantics.org/jenkins/job/SDK/job/Simantics%20SDK/) build with parameters:
  * **GERRIT_REFNAME:** `release/x.y.z[.w]`
  * **PUBLISH_ARTIFACTS:** `true`

Running these two builds will ensure that both the external components required to build the SDK and the Simantics SDK for the new release branch are published online at `http://www.simantics.org/download/release/x.y.z[.w]/`.

After this, whenever changes are pushed/merged to `release/x.y.z[.w]` branch in Gerrit, new **SDK/Simantics SDK** builds are triggered automatically and they will publish the results at the same location online.

This means that one does not have to do any tricks after this to build and publish the SDK as a P2 repository online. It is an automated process that is performed by the [SDK/Simantics SDK](https://www.simantics.org/jenkins/job/SDK/job/Simantics%20SDK/) Jenkins job.

## Review documentation

Documentation to review:
* [Developer wiki](http://dev.simantics.org/)
* [End-user wiki](https://www.simantics.org/end_user_wiki)
* [Member wiki](https://www.simantics.org/members)

For each wiki page:
* Read through and get authors to fix found problems, such as TODOs or invalid information.

## Review tutorials

* Ensure tutorial wiki documentation at http://dev.simantics.org/index.php/Tutorials is up-to-date with the released platform
* Ensure tutorial projects and product build properly

* com.acme.movie
  - Build with Buckminster, com.acme.movie.product.site.feature

## Tag release/* branches

When the release branches are ready for the release, tag them with the tag `vx.y.z[.w]`:

    git clone ssh://<user>@www.simantics.org:29418/simantics/platform.git
    cd platform    
    git checkout release/x.y.z[.w]
    git tag vx.y.z[.w] -m "Simantics x.y.z[.w] release"
    git push origin --tags

    git clone ssh://<user>@www.simantics.org:29418/simantics/third-party.git
    cd third-party
    git checkout release/x.y.z[.w]
    git tag vx.y.z[.w] -m "Simantics x.y.z[.w] release"
    git push origin --tags

> Note The -m argument must be supplied to create an [annotated tag](https://git-scm.com/book/en/v2/Git-Basics-Tagging).
> Only annotated or signed tags can be pushed to Gerrit.

## Backup documentation wiki databases 

This step is only necessary for major/minor releases, not for service releases.

The wiki databases to be backed up are:
* [Developer wiki](http://dev.simantics.org/)
* [End-user wiki](https://www.simantics.org/end_user_wiki)

These are MediaWiki installations. The only sane way to "tag" the documentation
is to back up the mysql database backing the wiki. Should the wiki be required
at a later time for some reason, we'll put the documentation up then in a
separate Mediawiki installation. 

1. Dump documentation wiki databases using [dump-wikis.sh](./dump-wikis.sh) script.
2. Put the generated backup x.y.z.tar.gz at /var/backup/simantics-releases/x.y.z/wiki/

## Compile change log entry

* Edit the [main page](https://www.simantics.org/redmine/projects/simantics-platform/wiki/ChangeLog) and add a a link for release x.y.z[.w].
* Open the change log page of the previous release, e.g. [1.25.0](https://www.simantics.org/redmine/projects/simantics-platform/wiki/Simantics_1250)
* Open the new link in another browser/tab to start editing the new wiki page
* Edit the previous release's page and copy its wiki source contents over to the new release's page.
* Fix the content to match the new release:
  * Remove the issue content of the previous release
  * Fix release number, release date, release branch link and the link to all issues closed for the release
    * Save new public issue query that lists all issues closed for the release by [starting from the previous release's query](https://www.simantics.org/redmine/projects/simantics-platform/issues?query_id=122)
  * Add filter to query: **Release Notes: Any** to only show issues that have some content in their Release Notes field
  * Export closed issue list as CSV with selected columns only. Open the resulting CSV file in Excel.
  * Use **Data -> Text to Columns** with tab column separation to columnize the result
  * Format the list as a table so that there is only one issue / row
  * Remove all other columns besides: Issue #, Tracker, Release Notes
  * Format the data into a Textile table:
    * Copy to table contents as text into [PSPad](http://www.pspad.com/)
    * Replace (CTRL+H) tabs (`\t`) with `|` with `Regular Expressions` selection checked
    * Use **Insert Text Into Lines..** (ALT-I) to fix line beginnings and ends:
      * Text:
        * At Lines Begin: `|#`
        * At Lines End: `|`
  * Copy the resulting textile table over to the change log page
* Highlight major issues in the list by changing the text background color of the **Type** column:
  * `%{background: lightsalmon}Major Bug%`
  * `%{background: lightgreen}Major Feature%`
  * `%{background: lightgreen}Major Enhancement%`

## Disseminate information about the release

* [Developer Wiki](http://dev.simantics.org): Update roadmap at http://dev.simantics.org/index.php/Roadmap
* [Redmine](https://www.simantics.org/redmine/): Post news on the developer/user-visible changes here.
* [simantics.org](https://www.simantics.org): Post news on the release and a link to the redmine post
* [Members Wiki](https://www.simantics.org/members/): Update frame plan to reflect the realized dates and link to Redmine news
* [mailto:simantics-developers@simantics.org](mailto:simantics-developers@simantics.org) Send "newsletter" to `simantics-developers@simantics.org:

**Newsletter template:**
~~~
Hello everyone,
   
Simantics release x.y.z[.w] has been released. Head over to
https://www.simantics.org/redmine/news/<news number>
for the release news.

Best regards,
Simantics Release Engineering Team
~~~

**Redmine news template:**
~~~
Title: Simantics x.y.z[.w] released

Simantics x.y.z[.w] was released on <date>.
Please find change log at: [[simantics-platform:Simantics_xyzw|Simantics x.y.z[.w]]]

Insert some general thoughts on the release...
~~~

----

# TODO

* Create a parametrized release train pipeline build in Jenkins that creates all artifacts of a simantics release
  * Desktop, Sysdyn, R, Simupedia, FMIL, FMI Studio


* Incorporate tutorial code in the platform repository as a separate folder to allow platform builds to directly ensure that the tutorial code still builds OK
 
* Start using https://github.com/mbarbero/fr.obeo.releng.targetplatform to generate `.target` files. `.tpd` files allow specifying version ranges instead of specific versions.

